/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package gui.run;


import gui.ClosableJFrame;
import gui.layouts.DialogLayout;

import javax.swing.*;
import java.awt.*;

import addbk.print.labels.LabelBean;


public abstract class RunLabelPanel
        extends JPanel implements Runnable {

    private LabelBean labelBean = LabelBean.restore();


    public RunLabelPanel() {

        init();
    }

    private void init() {
        setLayout(new BorderLayout());
        add(getLabelMarginPanel(), BorderLayout.CENTER);
        add(getButtonPanel(), BorderLayout.SOUTH);

    }

    private JPanel getLabelMarginPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new GridLayout(2, 0));
        jp.add(getLabelPanel());
        jp.add(getMarginPanel());
        return jp;
    }

    private JPanel getLabelPanel() {
        JPanel jp = new JPanel();
        jp.setBorder(BorderFactory.createTitledBorder("Label Dimensions in mm"));
        jp.add(new JLabel("Number of Labels per Column"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getNumberOfLabelsPerColumn(), 1, 100, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setNumberOfLabelsPerColumn(i);
            }
        });
        jp.add(new JLabel("Number of Labels"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getNumberOfLabelsPerSheet(), 1, 100, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setNumberOfLabelsPerSheet(i);
            }
        });
        jp.add(new JLabel("Vertical Gutter"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getVerticalGutterInMM(), 0, 50, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setVerticalGutterInMM(i);
            }
        });
        jp.add(new JLabel("Horizontal Gutter"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getHorizontalGutterInMM(), 0, 50, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setHorizontalGutterInMM(i);
            }
        });
              jp.add(new JLabel("Label Width"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getLabelWidthInMM(), 0, 1000, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setLabelWidthInMM(i);
            }
        });
              jp.add(new JLabel("Label Height"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getLabelHeightInMM(), 0, 1000, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setLabelHeightInMM(i);
            }
        });
        jp.setLayout(new DialogLayout());
        return jp;
    }

    private JPanel getMarginPanel() {
        JPanel jp = new JPanel();
        jp.setLayout(new DialogLayout());
        jp.setBorder(BorderFactory.createTitledBorder("Page Margins in mm"));
        jp.add(new JLabel("Left"));

        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getPageMarginLeftInMM(), 1, 100, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setPageMarginLeftInMM(i);
            }
        });

        jp.add(new JLabel("Right"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getPageMarginRightInMM(), 1, 100, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setPageMarginRightInMM(i);
            }
        });
        jp.add(new JLabel("Top"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getPageMarginTopInMM(), 1, 100, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setPageMarginTopInMM(i);
            }
        });
        jp.add(new JLabel("Bottom"));
        jp.add(new RunSpinner(new SpinnerNumberModel(labelBean.getPageMarginBottomInMM(), 1, 100, 1)) {
            public void run() {
                int i = ((Integer) getValue()).intValue();
                labelBean.setPageMarginBottomInMM(i);
            }
        });


        return jp;
    }


    public addbk.print.labels.LabelBean getValue() {
        return labelBean;
    }


    private JPanel getButtonPanel() {
        JPanel jp = new JPanel();

        jp.setLayout(new FlowLayout());
        jp.add(new RunButton("ok") {
            public void run() {
                RunLabelPanel.this.run();
                labelBean.save();
            }
        });
        return jp;
    }


    public static void main(String[] args) {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.add(new RunLabelPanel() {
            public void run() {
                System.out.println(getValue());
            }
        });
        cf.pack();
        cf.setVisible(true);
    }


}