package security;

import classUtils.javassist.CannotCompileException;
import classUtils.javassist.NotFoundException;
import classUtils.putils.ClassPathUtils;
import gui.In;
import gui.dialogs.BeanDialog;
import utils.CompactJava;
import xml.Utils;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.security.KeyStore;
import java.util.prefs.Preferences;

/**
 * Copyright DocJava, inc. User: lyon Date: Sep 3, 2004 Time: 9:12:18 AM
 */
public class WebStartBean extends KeyStoreBean implements Serializable {
    private String remoteSystemPassword = "";
    private String codeBase = "";
    private String iconUrl = "";
    private String hostUrl = "";
    private String rootDirectory = "";
    private String className = "";
    private String webServer = "";
    private String webServerUid = "";
    private final static String key = "WebStartBean";

    public WebStartBean() {
    }

    public static WebStartBean fromXml(String xmlString) {
        return (WebStartBean) Utils.decodeXml(xmlString);
    }

    public String toXml() {
        return CompactJava.toXml(this) + "</java>";
    }

    public String toString() {
        return "\npassword=" +
                remoteSystemPassword +
                "\ncodeBase=" +
                codeBase +
                "\niconUrl=" +
                iconUrl +
                "\nhostUrl=" +
                hostUrl +
                "\nrootDirectory=" +
                rootDirectory +
                "\nclassName=" +
                className +
                "\nwebServer=" +
                webServer +
                "\nwebServerUid=" + webServerUid;
    }

    public String getCodeBase() {
        return codeBase;
    }

    public void setCodeBase(String codeBase) {
        this.codeBase = codeBase;
    }

    public String getIconUrl() {
        return iconUrl;
    }

    public void setIconUrl(String iconUrl) {
        this.iconUrl = iconUrl;
    }

    public String getHostUrl() {
        return hostUrl;
    }

    public void setHostUrl(String hostUrl) {
        this.hostUrl = hostUrl;
    }

    public String getRootDirectory() {
        return rootDirectory;
    }

    public void setRootDirectory(String rootDirectory) {
        this.rootDirectory = rootDirectory;
    }

    public String getClassName() {
        return className;
    }

    public void setClassName(String className) {
        this.className = className;
    }

    public String getWebServer() {
        return webServer;
    }

    public void setWebServer(String webServer) {
        this.webServer = webServer;
    }

    public String getWebServerUid() {
        return webServerUid;
    }

    public void setWebServerUid(String webServerUid) {
        this.webServerUid = webServerUid;
    }


    public void save() {
        String s = toXml();
        Preferences p = Preferences.userRoot();
        p.put(key, s);
    }

    public static WebStartBean getFromPreferences() {
        Preferences p = Preferences.userRoot();
        String s = p.get(key, null);
        if (s == null) return
                WebStartBean.getDefaultWebStartBean();
        return fromXml(s);
    }

    public static void main(String[] args) {
        //testWebStartBean();
        WebStartBean w2 = WebStartBean.getFromPreferences();
        System.out.println("read from preferences" + w2.toXml());
    }

    public static void testWebStartBean() {
        WebStartBean wsb = new WebStartBean();
        wsb.setAlias("docjava");
        wsb.setClassName("security.WebStartBean");
        wsb.setVendor("DocJava, Inc");
        final String x = wsb.toXml();
        System.out.println(x);
        wsb.save();
    }

    /**
     * Get WebStartBean and use a GUI to prompt the USER.
     *
     * @return the defaul webstart bean from prefrences
     */
    public static WebStartBean getDefaultWebStartBeanGUI() {
        final WebStartBean wsb = WebStartBean.getFromPreferences();
        BeanDialog dialog = new BeanDialog(wsb);
        dialog.pack();
        dialog.setVisible(true);
        wsb.save();
        Object o = dialog.getData();

        return (WebStartBean) o;
    }

    public static WebStartBean getDefaultWebStartBean() {
        WebStartBean wsb = new WebStartBean();
        String password = In.getPassword("enter keystore password");
        wsb.setRemoteSystemPassword(password);

        File ksf = KeyUtils.getKeystoreFile();
        wsb.setKeyStoreFile(ksf.toString());
        KeyStore ks = KeyUtils.getKeyStore(ksf, password);
        try {
            wsb.setClassName(ClassPathUtils.getClassFromFile().getName());
            wsb.setRootDirectory("/var/www/html/book/cgij/code/jnlp/");
            wsb.setWebServer("192.168.1.95");
            wsb.setWebServerUid("lyon");
            wsb.setIconUrl("http://show.docjava.com.com:8086/" +
                    "consulti/docjava.jpe\"");
            wsb.setCodeBase("http://show.docjava.com.com:8086/" +
                    "book/cgij/code/jnlp/");
            wsb.setAlias(KeyUtils.getAlias(ks));
            wsb.setVendor("DocJava, Inc.");
            wsb.setJarFileName(wsb.getClassName() + ".jar");
            wsb.setHostUrl("http://www.docjava.com");
        } catch (NotFoundException e) {
            In.message(e);

        } catch (IOException e) {
            In.message(e);

        } catch (CannotCompileException e) {
            In.message(e);

        }
        return wsb;
    }

    public String getRemoteSystemPassword() {
        return remoteSystemPassword;
    }

    public void setRemoteSystemPassword(String remoteSystemPassword) {
        this.remoteSystemPassword = remoteSystemPassword;
    }

}
