package video;

import gui.icons.Icons;
import video.jmf.CaptureUtil;

import javax.media.*;
import javax.media.control.FrameGrabbingControl;
import javax.media.format.VideoFormat;
import javax.media.util.BufferToImage;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.Date;
import java.net.URL;
import java.net.MalformedURLException;

import j2d.ImageUtils;

/**
 * Created by IntelliJ IDEA. User: dlyon Date: Apr 13, 2005 Time: 5:10:33
 * PM Copyright DocJava, 2005
 */
public class CameraUtils {
    // CameraUtils.getImageFromCamera
    public static BufferedImage getImageFromCamera()
            throws IOException, NoPlayerException, CannotRealizeException,
                   InterruptedException {
        // Create capture device

        Player player = getPlayer();
        if (player==null) return
                ImageUtils.getBufferedImage(
                        ImageUtils.getImage(Icons.xImage));
        BufferedImage imageFromCamera = getImageFromCamera(player);
        close(player);
        return imageFromCamera;

    }

    public static Player getPlayer() throws IOException,
                                            NoPlayerException,
                                            CannotRealizeException,
                                            InterruptedException {

        //CaptureDeviceInfo deviceInfo = CaptureDeviceManager.getDevice(
        //        "vfw:Microsoft WDM Image Capture (Win32):0");
        CaptureDeviceInfo deviceInfo =  CaptureUtil.getDevice();
        if (deviceInfo == null) return null;
        Player player = Manager.createRealizedPlayer(
                deviceInfo.getLocator());
        player.start();

        // Wait a few seconds for camera to initialise (otherwise img==null)
        Thread.sleep(2500);
        return player;
    }

    public static BufferedImage getImageFromCamera(Player player) {
        if (player == null) return getNoCameraImage();
        // Grab a frame from the capture device
        FrameGrabbingControl frameGrabber = (FrameGrabbingControl)
                player.getControl(
                        "javax.media.control.FrameGrabbingControl");
        Buffer buf = frameGrabber.grabFrame();

        // Convert frame to an buffered image so it can be processed and saved
        Image img = (
                new BufferToImage((VideoFormat) buf.getFormat()).createImage(
                        buf));
        BufferedImage buffImg = new BufferedImage(
                img.getWidth(null), img.getHeight(null),
                BufferedImage.TYPE_INT_RGB);
        Graphics2D g = buffImg.createGraphics();
        g.drawImage(img, null, null);

        // Overlay curent time on image
        g.setColor(Color.RED);
        g.setFont(new Font("Verdana", Font.BOLD, 16));
        g.drawString((new Date()).toString(), 10, 25);

        // Save image to disk as PNG
        return buffImg;
    }

    private static BufferedImage getNoCameraImage() {
        BufferedImage bi = new BufferedImage(64, 64,
                BufferedImage.TYPE_INT_RGB);
        Graphics2D g = bi.createGraphics();
        g.setColor(Color.RED);
        g.setFont(new Font("Verdana", Font.BOLD, 16));
        g.drawString("no camera", 10, 25);
       return bi;
    }

    public static void close(Player player) {
        // Stop using webcam
        player.close();
        player.deallocate();
    }
    public static Image getWebCamImage() throws MalformedURLException {
        URL url = new URL("http://172.16.11.191/IMAGE.JPG");
        return ImageUtils.getImage(url);
    }
    public static void main(String[] args) throws MalformedURLException {
        System.out.println("start");
        Image img1 = getWebCamImage();
        Image img2 = getWebCamImage();
        Image imagea=null;
        for (int i=0; i < 10; i++) {
             imagea = ImageUtils.average(img1, img2);
            Image img3 = getWebCamImage();
            img1 = imagea;
            img2 = img3;
            System.out.println("image i:"+i);
        }
        ImageUtils.displayImage(imagea,"camera");
    }
}
